@@ -4,7 +4,7 @@
 
 ;; Author: Adam Porter <adam@alphapapa.net>
 ;; URL: https://github.com/alphapapa/burly.el
-;; Version: 0.2
+;; Version: 0.3
 ;; Package-Requires: ((emacs "27.1") (map "2.1"))
 ;; Keywords: convenience
 
@@ -155,7 +155,7 @@ a project."
 
 ;;;###autoload
 (defun burly-kill-frames-url ()
-  "Copy current frameset's URL to the kill ring."
+  "Copy current frameset URL to the kill ring."
   (interactive)
   (let ((url (burly-frames-url)))
     (kill-new url)
@@ -228,8 +228,14 @@ a project."
     (pcase-exhaustive subtype
       ("bookmark" (burly--bookmark-url-buffer urlobj))
       ("file" (burly--file-url-buffer urlobj))
-      ("name" (get-buffer (decode-coding-string (cdr (url-path-and-query urlobj))
-						'utf-8-unix))))))
+      ("name" (let ((buffer-name (decode-coding-string (cdr (url-path-and-query urlobj))
+						       'utf-8-unix)))
+                (or (get-buffer buffer-name)
+                    (with-current-buffer (get-buffer-create (concat "*Burly (error): " buffer-name "*"))
+                      (insert "Burly was unable to get a buffer named: " buffer-name "\n"
+                              "URL: " url "\n"
+                              "Please report this error to the developer\n\n")
+                      (current-buffer))))))))
 
 (defun burly-buffer-url (buffer)
   "Return URL for BUFFER."
@@ -240,12 +246,12 @@ a project."
                    (when-let* ((record (ignore-errors
                                          (bookmark-make-record))))
 		     (cl-labels ((encode (element)
-					 (cl-typecase element
-					   (string (encode-coding-string element 'utf-8-unix))
-					   (proper-list (mapcar #'encode element))
-					   (cons (cons (encode (car element))
-						       (encode (cdr element))))
-					   (t element))))
+				   (cl-typecase element
+				     (string (encode-coding-string element 'utf-8-unix))
+				     ((satisfies proper-list-p) (mapcar #'encode element))
+				     (cons (cons (encode (car element))
+						 (encode (cdr element))))
+				     (t element))))
 		       ;; Encode all strings in record with UTF-8.
 		       ;; NOTE: If we stop using URLs in the future, maybe this won't be needed.
 		       (setf record (encode record)))
@@ -307,7 +313,9 @@ FRAMES defaults to all live frames."
     (setf (frameset-states frameset)
           (cl-loop for (frame-parameters . window-state) in (frameset-states frameset)
                    collect (cons frame-parameters (burly--bufferize-window-state window-state))))
-    (frameset-restore frameset)))
+    (condition-case err
+        (frameset-restore frameset)
+      (error (delay-warning 'burly (format "Error while restoring frameset: ERROR:%S  FRAMESET:%S" err frameset))))))
 
 ;;;;; Windows
 
@@ -364,21 +372,22 @@ If NULLIFY, set the parameter to nil."
 
 (defun burly--bufferize-window-state (state)
   "Return window state STATE with its buffers reincarnated."
-  (cl-labels ((bufferize-state
-               ;; Set windows' buffers in STATE.
-               (state) (pcase state
-                         (`(leaf . ,_attrs) (bufferize-leaf state))
-                         ((pred atom) state)
-                         (`(,_key . ,(pred atom)) state)
-                         ((pred list) (mapcar #'bufferize-state state))))
-              (bufferize-leaf
-               (leaf) (pcase-let* ((`(leaf . ,attrs) leaf)
-                                   ((map parameters buffer) attrs)
-                                   ((map burly-url) parameters)
-                                   (`(,_buffer-name . ,buffer-attrs) buffer)
-                                   (new-buffer (burly-url-buffer burly-url)))
-                        (setf (map-elt attrs 'buffer) (cons new-buffer buffer-attrs))
-                        (cons 'leaf attrs))))
+  (cl-labels ((bufferize-state (state)
+                "Set windows' buffers in STATE."
+                (pcase state
+                  (`(leaf . ,_attrs) (bufferize-leaf state))
+                  ((pred atom) state)
+                  (`(,_key . ,(pred atom)) state)
+                  ((pred list) (mapcar #'bufferize-state state))))
+              (bufferize-leaf (leaf)
+                "Recreate buffers in LEAF."
+                (pcase-let* ((`(leaf . ,attrs) leaf)
+                             ((map parameters buffer) attrs)
+                             ((map burly-url) parameters)
+                             (`(,_buffer-name . ,buffer-attrs) buffer)
+                             (new-buffer (burly-url-buffer burly-url)))
+                  (setf (map-elt attrs 'buffer) (cons new-buffer buffer-attrs))
+                  (cons 'leaf attrs))))
     (if-let ((leaf-pos (cl-position 'leaf state)))
         ;; A one-window frame: the elements following `leaf' are that window's params.
         (append (cl-subseq state 0 leaf-pos)
@@ -405,8 +414,14 @@ from the hook."
 ;;;###autoload
 (defun burly-bookmark-handler (bookmark)
   "Handler function for Burly BOOKMARK."
-  (burly-open-url (alist-get 'url (bookmark-get-bookmark-record bookmark)))
-  (setf burly-opened-bookmark-name (car bookmark)))
+  (let ((previous-name burly-opened-bookmark-name))
+    ;; Set opened bookmark name before actually opening it so that the
+    ;; tabs-mode advice functions can use it beforehand.
+    (setf burly-opened-bookmark-name (car bookmark))
+    (condition-case err
+	(burly-open-url (alist-get 'url (bookmark-get-bookmark-record bookmark)))
+      (error (setf burly-opened-bookmark-name previous-name)
+	     (signal (car err) (cdr err))))))
 
 (defun burly--bookmark-record-url (record)
   "Return a URL for bookmark RECORD."
@@ -435,20 +450,26 @@ URLOBJ should be a URL object as returned by
                                for value = (pcase key
                                              ('handler (intern (cadr prop)))
                                              ('help-args (read (cadr prop)))
-                                             ('help-fn (read (cadr prop)))
+                                             ('help-fn (ignore-errors
+                                                         ;; NOTE: Due to changes in help-mode.el which serialize natively
+                                                         ;; compiled subrs in the bookmark record, which cannot be read
+                                                         ;; back (which actually break the entire bookmark system when
+                                                         ;; such a record is saved in the bookmarks file), we have to
+                                                         ;; workaround a failure to read here.  See bug#56643.
+                                                         (read (cadr prop))))
                                              ('position (cl-parse-integer (cadr prop)))
                                              (_ (read (cadr prop))))
                                collect (cons key value)))
                (record (cons path props)))
     (cl-labels ((decode (element)
-			(cl-typecase element
-			  (string (decode-coding-string element 'utf-8-unix))
-			  (proper-list (mapcar #'decode element))
-			  (cons (cons
-				 (decode (car element))
-				 (decode (cdr element))))
-			  (t element))))
-      ;; Decode all strings in record with UTF-8.
+                  "Return ELEMENT with strings decoded using `utf-8-unix'."
+		  (cl-typecase element
+		    (string (decode-coding-string element 'utf-8-unix))
+		    ((satisfies proper-list-p) (mapcar #'decode element))
+		    (cons (cons
+			   (decode (car element))
+			   (decode (cdr element))))
+		    (t element))))
       ;; NOTE: If we stop using URLs in the future, maybe this won't be needed.
       (setf record (decode record)))
     (save-window-excursion
@@ -459,6 +480,7 @@ URLOBJ should be a URL object as returned by
 
 (defun burly-bookmark-names ()
   "Return list of all Burly bookmark names."
+  (bookmark-maybe-load-default-file)
   (cl-loop for bookmark in bookmark-alist
            for (_name . params) = bookmark
            when (equal #'burly-bookmark-handler (alist-get 'handler params))
@@ -532,25 +554,20 @@ URLOBJ should be a URL object as returned by
   "In BUFFER, jump to heading and position from QUERY, and return a buffer.
 If QUERY specifies that the buffer should be indirect, a new,
 indirect buffer is returned.  Otherwise BUFFER is returned."
-  ;; `pcase's map support uses `alist-get', which does not work with string keys
-  ;; unless its TESTFN arg is bound to, e.g. `equal', but `map-elt' has deprecated
-  ;; its TESTFN arg, and there's no way to pass it or bind it when using `pcase'
-  ;; anyway.  So we rebind `alist-get' to a function that uses `assoc-string'.
   (with-current-buffer buffer
-    (cl-letf (((symbol-function 'alist-get)
-               (lambda (key alist &optional _default _remove _testfn)
-                 ;; Only the first value in the list of values is returned, so multiple
-                 ;; values are not supported.  I don't expect this to be a problem...
-                 (cadr (assoc-string key alist)))))
-      (pcase-let* (((map ("pos" pos)
-                         ("indirect" indirect)
-                         ("narrowed" narrowed)
-                         ("top-olp" top-olp)
-                         ("point-olp" point-olp)
-                         ("relative-pos" relative-pos))
-                    query)
-                   (heading-pos (when top-olp
-                                  (org-find-olp (read top-olp) 'this-buffer))))
+    ;; Because the query is built with `url-build-query-string', which allows keys
+    ;; to have multiple values, we define `query-elt', which handles that and only
+    ;; returns the first value (which is fine since we only use single values).
+    (cl-labels ((query-elt (query elt)
+                  (car (map-elt query elt))))
+      (let* ((pos (query-elt query "pos"))
+             (indirect (query-elt query "indirect"))
+             (narrowed (query-elt query "narrowed"))
+             (top-olp (query-elt query "top-olp"))
+             (point-olp (query-elt query "point-olp"))
+             (relative-pos (query-elt query "relative-pos"))
+             (heading-pos (when top-olp
+                            (org-find-olp (read top-olp) 'this-buffer))))
         (widen)
         (if heading-pos
             (goto-char heading-pos)
@@ -563,7 +580,6 @@ indirect buffer is returned.  Otherwise BUFFER is returned."
           (forward-char (string-to-number relative-pos)))
         (current-buffer)))))
 
-
 ;;;; Footer
 
 (provide 'burly)
